(*-------------------------------FileWorks.pas--------------------------
 V1.0.236 - 26.12.2001 changed ListFilesRecursive:
                          1. previous version has a bug in recursive search
                             related to DeleteFilesRecursive
                          2. added ShowPath functionality

 V1.0.235 - 19.12.2001 added SaveStringToFile
                       added ReadStringFromFile
------------------------------------------------------------------------
*)

unit FileWorks;

interface

uses Windows, Classes, SysUtils, ShellAPI, StringWorks, FileCtrl;

function DeleteDirectory(Directory: String; CanUndo, DeleteNotEmpty, GUI: Boolean): Boolean;
function DeleteFiles(Path, Mask: string): Boolean;
function DeleteFilesRecursive(Path, Mask: string): Boolean;
function DirIsEmpty(Path: String): Boolean;
function ExtractDriveFromPath(Path: String): String;
function ExtractPureFilename(Name: String): String;
function ExtractFileExtension(Name: String): String;
function FileCopy(Source, Destination: String): Integer;
function GetFileSize(Datei: String): Int64;
function ITB(Directory: String): String;
function ListFiles(Path, Mask: string; ShowPath: boolean): TStringlist;
function ListFilesRecursive(Path, Mask: string; ShowPath: Boolean): TStringList;
//function ListFilesRecursive2(Path, Mask: String; ShowPath: Boolean): TStringList;
function ListFolders(Path: string; ShowPath: boolean): TStringlist;
function RenameDirectory(OldPath, NewPath: String): Boolean;
function MoveDirectory(Directory, DestinationFolder: String): Boolean;
function ReadStringFromFile(var Filename: String): String;
procedure SaveStringToFile(var Str, Filename: String);



implementation

function ExtractPureFilename(Name: String): String;
var
   Dots, DotPos: Integer;
begin
   Dots:= StringCountInStr('.', Name);
   DotPos:= SubPositionByIndex(Name, '.', Dots);
   result:= StrLeft(Name, DotPos - 1);
end;

function ExtractFileExtension(Name: String): String;
var
   Dots, DotPos, StrLn: Integer;
begin
   Dots:= StringCountInStr('.', Name);
   StrLn:= StrLen(PChar(Name));
   DotPos:= SubPositionByIndex(Name, '.', Dots);
   result:= Copy(Name, DotPos + 1, StrLn);
end;

function FileCopy(Source, Destination: String): Integer;
var
  Q, Z: TFileStream;
begin
   if not FileExists(Source) then begin
      result:= -2; (* Fehlercode fr "Quelle nicht gefunden" *)
      exit;
   end;
   if FileExists(Destination) then begin
      result:= -3; (* Fehlercode fr "Zieldatei vorhanden" *)
      exit;
   end;
   try
      Q := TFileStream.Create(Source, fmOpenRead);
      Z := TFileStream.Create(Destination, fmCreate);
      Z.CopyFrom(Q, Q.Size);
      Q.Free;
      Z.Free;
      result:= 0; (* Fehlercode 0 fr erfolgreiche Ausfhrung *)
   except
      result:= -4; (* Fehlercode fr "Datei konnte nicht kopiert werden" *)
   end;
end;

function FileMove(Source, Destination: String): Integer;
var
  Q, Z: TFileStream;
begin
   if not FileExists(Source) then begin
      result:= -2; (* Fehlercode fr "Quelle nicht gefunden" *)
      exit;
   end;
   if FileExists(Destination) then begin
      result:= -3; (* Fehlercode fr "Zieldatei vorhanden" *)
      exit;
   end;
   try
      Q := TFileStream.Create(Source, fmOpenRead);
      Z := TFileStream.Create(Destination, fmCreate);
      Z.CopyFrom(Q, Q.Size);
      Q.Free;
      Z.Free;
      if not DeleteFile(PChar(Source)) then begin
         result:= -5; (* Fehlercode fr "Quelldatei kann nicht entfernt werden" *)
         exit;
      end;
   result:= 0; (* Fehlercode 0 fr erfolgreiche Ausfhrung *)
   except
      result:= -4; (* Fehlercode fr "Datei konnte nicht kopiert werden" *)
   end;
end;

function ListFiles(Path, Mask: string; ShowPath: boolean): TStringlist;
var
  SRec: TSearchRec;
  SL: TStringList;
begin
   (* ++++++++++++++++ Code sample by Pumi ++++++++++++++++++ *)
   SL := TStringList.Create;
   FindFirst(Path + Mask, not faDirectory, SRec);
   if ShowPath then
      SL.Add(Path + SRec.Name)
   else
      SL.Add(SRec.Name);
   while FindNext(SRec) = 0 do
      if ShowPath then
         SL.Add(Path + SRec.Name)
      else
         SL.Add(SRec.Name);
   FindClose(SRec);
   Result := SL;
end;

function ListFolders(Path: string; ShowPath: boolean): TStringlist;
var
  SRec: TSearchRec;
  SL: TStringList;
begin
   (* ++++++++++++++++ Code sample by Pumi ++++++++++++++++++ *)
   SL := TStringList.Create;
   FindFirst(Path + '*.', not faAnyFile + not faSysFile + not faHidden + not faReadOnly, SRec);
   if ShowPath then
      SL.Add(Path + SRec.Name)
   else
      SL.Add(SRec.Name);
   while FindNext(SRec) = 0 do
      if ShowPath then
         SL.Add(Path + SRec.Name)
      else
         SL.Add(SRec.Name);
   FindClose(SRec);
   Result := SL;
end;

function DeleteFiles(Path, Mask: string): Boolean;
var
  SRec: TSearchRec;
  SL: TStringList;
  I: Integer;
begin
   (* ++++++++++++++++ Code based on Assarbad ++++++++++++++++++ *)
   result:= FALSE;
   SL := TStringList.Create;
   FindFirst(Path + Mask, not faDirectory, SRec);
   SL.Add(Path + SRec.Name);
   while FindNext(SRec) = 0 do SL.Add(Path + SRec.Name);
   FindClose(SRec);
   if SL.Count < 1 then exit;
   for I:= 0 to SL.Count - 1 do begin
      DeleteFile(PChar(SL[I]));
   end;
   result:= TRUE;
end;

{ -------- This version has a bug in recursive search---------
function ListFilesRecursive(Path, Mask: string; ShowPath: Boolean): TStringList;
var SR: TSearchRec;
    err, I: Integer;
    dummy: String;
    TempList: TStringList;
begin
   (* ++++++++++++++++ Code sample by Assarbad ++++++++++++++++++ *)
     err:=FindFirst(Path+Mask, faAnyFile, SR);
     Path:= IncludeTrailingBackslash(Path);
     result:= TStringList.Create;
     TempList:= TStringList.Create;
     while err=0 do begin
           if (sr.name<>'.') and (sr.name<>'..') then
              if ((sr.Attr and fadirectory)<>0) then begin
                 dummy:=path+sr.name+'\';
                 TempList.Assign(ListFilesRecursive(dummy, mask, ShowPath));
                 for I:= 0 to TempList.Count - 1 do begin
//                    if ShowPath then result.Add(Dummy + TempList[I]) else
                       result.Add(TempList[I]);
                 end;
              end else if ShowPath then result.Append(Path + sr.name) else
                 result.Append(sr.name);
           err:=findnext(sr);
     end;
     FindClose(SR);
     TempList.Free;
end;}

function ListFilesRecursive(Path, Mask: String; ShowPath: Boolean): TStringList;
var SR: TSearchRec;
    Erg, I: Integer;
    RelDir, TempStr : String;
    TempList: TStringList;
begin
   IncludeTrailingBackslash(Path);
   RelDir := ExtractFilePath(Mask);
   Mask := ExtractFileName(Mask);
   result:= TStringList.Create;
   TempList:= TStringList.Create;
   Erg := FindFirst(Path+RelDir+'*.*', faDirectory, SR);
   while Erg = 0 do
   begin
      if (SR.attr and faDirectory) <> 0 then
         if SR.Name[1] <> '.' then begin
            TempList.Assign(ListFilesRecursive2(Path, RelDir+SR.Name+'\'+Mask, ShowPath));
            for I:= 0 to TempList.Count - 1 do begin
               TempStr:= TempList[I];
               if ShowPath and not (TempStr[2] = ':') then TempStr:= Path + TempStr;
               result.Add(TempList[I]);
            end;
         end;
      Erg := FindNext(SR);
   end;
   Erg := FindFirst(Path+RelDir+Mask, $27, SR);
   {Das Attribut $27 bedeutet
   alle echten Dateien}
   while Erg = 0 do
   begin
      if not ShowPath then
         result.Add(RelDir+SR.Name)
      else
      result.Add(Path+RelDir+SR.Name);
      Erg := FindNext(SR);
   end;
   FindClose(SR);
   TempList.Free;
end;

function DeleteFilesRecursive(Path, Mask: string): Boolean;
var SR: TSearchRec;
    err, I:integer;
    dummy:string;
    SL: TStringList;
begin
   (* ++++++++++++++++ Code based on Assarbad ++++++++++++++++++ *)
     result:= FALSE;
     err:=FindFirst(Path+Mask, faAnyFile, SR);
     SL:= TStringList.Create;
     while err=0 do begin
           if (sr.name<>'.') and (sr.name<>'..') then
              if ((sr.Attr and fadirectory)<>0) then begin
                 dummy:=path+sr.name+'\';
                 ListFilesRecursive(dummy, Mask, TRUE);
              end else SL.Append(sr.name);
           err:=findnext(sr);
     end;
     FindClose(SR);
     if SL.Count < 1 then exit;
     for I:= 0 to SL.Count - 1 do begin
        DeleteFile(PChar(SL[I]));
     end;
     result:= TRUE;
end;

function DirIsEmpty(Path: String): Boolean;
var
   SL: TStringList;
begin
   SL:= ListFilesRecursive(ITB(Path), '*.*', TRUE);
   result:= (SL.Count < 1);
end;

function ExtractDriveFromPath(Path: String): String;
begin
   if (StrLen(PChar(Path)) > 0) then result:= Copy(Path, 1, 1) else result:= '';
end;

function ITB(Directory: String): String;
begin
   result:= UpperCase(IncludeTrailingBackslash(Directory));
end;

function RenameDirectory(OldPath, NewPath: String): Boolean;
begin
   result:= FALSE;
   if not (ExtractDriveFromPath(OldPath) = ExtractDriveFromPath(NewPath)) then
      exit;
   result:= MoveFile(PChar(OldPath), PChar(NewPath));
end;

function MoveDirectory(Directory, DestinationFolder: String): Boolean;
var
  FOS : TSHFileOpStruct;
begin
   ZeroMemory(@FOS,SizeOf(FOS));
   with FOS do begin
      wFunc := FO_MOVE;
      fFlags := FOF_NOCONFIRMATION;
      pFrom := PChar(Directory + #0);
      pTo := PChar(DestinationFolder)
   end;
   RESULT := (0 = ShFileOperation(FOS));
end;

function DeleteDirectory(Directory: String; CanUndo, DeleteNotEmpty, GUI: Boolean): Boolean;
var
  FOS : TSHFileOpStruct;
  FOS_FLAG: DWORD;
begin
   result:= FALSE;
   Directory:= ITB(Directory);
   if not DirectoryExists(Directory) then exit;
   if not DeleteNotEmpty and not DirIsEmpty(Directory) then exit;
   ZeroMemory(@FOS,SizeOf(FOS));
   if CanUndo then FOS_FLAG:= FOF_ALLOWUNDO else FOS_FLAG:= 0;
   if not GUI then FOS_FLAG:= FOS_FLAG + FOF_NOCONFIRMATION;
   with FOS do begin
      wFunc := FO_DELETE;
      fFlags := FOS_FLAG;
      pFrom := PChar(Directory + #0);
   end;
   RESULT := (0 = ShFileOperation(FOS));
end;

function  GetFileSize(Datei: String): Int64;
var
   TempFileSize: Int64;
   SR: TSearchRec;
begin
     TempFileSize:= 0;
     if FindFirst(Datei,faAnyFile,SR) = 0 then
     TempFileSize:= SR.Size;
     FindClose(SR);
     result:= TempFileSize;
end;

procedure SaveStringToFile(var Str, Filename: String);
var
   TF: TextFile;
begin
   AssignFile(TF, Filename);
   Rewrite(TF);
   Write(TF, Str);
   Close(TF);
end;

function ReadStringFromFile(var Filename: String): String;
var
   TF: TextFile;
begin
   if not FileExists(Filename) then begin
      result:= '';
      exit;
   end;
   AssignFile(TF, Filename);
   Reset(TF);
   Read(TF, result);
   Close(TF);
end;


end.
