unit ProtectWorks;

interface

uses Classes, Forms, Dialogs, SysUtils, DirectDisk, StringWorks, HCMngr, Cipher,
     MSI_Machine, MSI_CPU, NumWorks, FileWorks, Windows;

const
   Letters: array[0..65] of Char = '+0123456789/ABCDEFGHIJKLMNOPQRSTUVWXYZ=abcdefghijklmnopqrstuvwxyz' + #0;
   NilString = '<ahoire>';
   NilInteger = -1;

type
  AUnusedSectors = Array[63..1232, 1235..3350] of Integer;

  TSectorArray = Array[0..(512*4)-1] of Byte;

  TLizenzRec = packed record
    Check:           Integer;
    Firma:           String;
    Vorname:         String;
    Name:            String;
    Zusatz:          String;
    Strasse:         String;
    PLZ:             String;
    Ort:             String;
    Land:            Integer;
    Produktindex:    Integer;
    Lizenzindex:     Integer;
    Lizenzanzahl:    Integer;
    Funktionsindex:  Integer;
    Key1:            Integer;
    Key2:            String;
  end;

  TLizenzRecCheck = (failLRCheck, failLRFirma, failLRVorname, failLRName,
                     failLRZusatz, failLRStrasse, failLRPLZ, failLROrt,
                     failLRLand, failLRProduktindex, failLRLizenzindex,
                     failLRLizenzanzahl, failLRFunktionsindex,
                     failLRKey1, failLRKey2);

  TLizenzRecChecks = set of TLizenzRecCheck;

  TSerialRec = packed record
    DiskID:          Integer;
    Date:            TDateTime;
  end;

  TGeneratorRec = packed record
    Check:           Integer;
    Firma:           String;
    Zusatz:          String;
    Strasse:         String;
    PLZ:             Integer;
    Ort:             String;
    Telefon:         String;
    Fax:             String;
    Email:           String;
    Kontakt:         String;
  end;

  TGeneratorRecCheck = (failGRCheck, failGRFirma, failGRZusatz, failGRStrasse,
                        failGRPLZ, failGROrt, failGRTelefon, failGRFax,
                        failGREmail, failGRKontakt);

  TGeneratorRecChecks = set of TGeneratorRecCheck;

  TFreeFunctionRec = packed record
    BaseFunctionality,
    ActiveSearch,
    Navigator,
    FullScreenView,
    TwainInterface: Boolean;
  end;

  TSystemRec = packed record
    BIOSDate,
    BIOSExtended,
    BIOSName,
    CPUSerialNumber,
    CPUVendorEx,
    CPUVendorID: String;
    CPUCount,
    CPUFamily,
    CPUFrequency,
    CPUModel,
    CPUStepping: Integer;
  end;

  TSysRecCheck = (failBIOSDate, failBIOSExtended, failBIOSName,
                  failCPUSerialNumber, failCPUVendorEx, failCPUVendorID,
                  failCPUCount, failCPUFamily, failCPUFrequency,
                  failCPUModel, failCPUStepping, failAllReady);

  TSysRecChecks = set of TSysRecCheck;

procedure ChangeStrInArray(var BArray: TSectorArray; Str: String; Offset: Cardinal);
function GetSector(Drive: Char; Sector: Integer): TSectorArray;
procedure SetSector(var SectorArray: TSectorArray; Drive: Char; Sector: Integer);
procedure RndSector(Drive: Char; Sector: Integer);
procedure RandomDriveSectors(MinSectors, MaxSectors: Integer; Drive: Char; Unused: AUnusedSectors);
function LizenzRec(Data: TSectorArray; Passwort: String): TLizenzRec;
function ReadLizenz(var LizenzRec: TLizenzRec; Drive: Char; Sector: Integer; Passwort: String): Integer;
function WriteLizenz(LizenzRec: TLizenzRec; Drive: Char; Sector: Integer; Passwort: String): Integer;
function GeneratorRec(Data: TSectorArray; Passwort: String): TGeneratorRec;
function ReadGenerator(var GeneratorRec: TGeneratorRec; Drive: Char; Sector: Integer; Passwort: String): Integer;
function WriteGenerator(GeneratorRec: TGeneratorRec; Drive: Char; Sector: Integer; Passwort: String): Integer;
function SystemRec: TSystemRec;
function DefaultSystemRec: TSystemRec;
function ReadSystem(var SystemRec: TSystemRec; Drive: Char; Sector: Integer; Passwort: String): Integer;
function ReadSystemFromFile(var SystemRec: TSystemRec; Filename, Passwort: String): String;
function WriteSystem(SystemRec: TSystemRec; Drive: Char; Sector: Integer; Passwort: String): Integer;
function WriteSystemToFile(SystemRec: TSystemRec; Filename, Passwort: String): Integer;

function CompareGeneratorRec(var GeneratorRec1, GeneratorRec2: TGeneratorRec): TGeneratorRecChecks;
function CompareLizenzRec(var LizenzRec1, LizenzRec2: TLizenzRec): TLizenzRecChecks;
function CompareSystemRec(var SystemRec1, SystemRec2: TSystemRec): TSysRecChecks;

implementation

procedure ChangeStrInArray(var BArray: TSectorArray; Str: String; Offset: Cardinal);//: TSectorArray;
var
   I: Cardinal;
   S: String;
begin
   Randomize;
   for I:= 0 to 1024 do begin
      S:= S + Letters[Random(Length(Letters))];
      BArray[I]:= Ord(S[I]);
   end;
   for I:= Offset to Offset + (StrLen(PChar(Str)) - 1) do begin
      BArray[I]:= Ord(Str[I-Offset+1]);
   end;
   BArray[Offset+StrLen(PChar(Str))]:= 0;
//   result:= BArray;
end;

function GetSector(Drive: Char; Sector: Integer): TSectorArray;
var
   DirDisk: TDirDisk;
begin
   DirDisk:= TDirDisk.Create(Application);
   DirDisk.NeedWriteWarn:= FALSE;
   DirDisk.DriveName:= Drive;
   DirDisk.FirstSector:= Sector;
   DirDisk.Sectors:= 1;
   DirDisk.ReadSectors(result);
   DirDisk.Free;
end;

procedure SetSector(var SectorArray: TSectorArray; Drive: Char; Sector: Integer);
var
   DirDisk: TDirDisk;
begin
   DirDisk:= TDirDisk.Create(Application);
   DirDisk.NeedWriteWarn:= FALSE;
   DirDisk.DriveName:= Drive;
   DirDisk.FirstSector:= Sector;
   DirDisk.Sectors:= 1;
   DirDisk.WriteSectors(SectorArray);
   DirDisk.Free;
end;

procedure RndSector(Drive: Char; Sector: Integer);
var
   SData: TSectorArray;
   I: Cardinal;
   S: String;
begin
   SData:= GetSector(Drive, Sector);
   Randomize;
   for I:= 0 to 1024 do begin
      S:= S + Letters[Random(Length(Letters))];
      SData[I]:= Ord(S[I]);
   end;
   SetSector(SData, Drive, Sector);
end;

procedure RandomDriveSectors(MinSectors, MaxSectors: Integer; Drive: Char;
            Unused: AUnusedSectors);
{var
   RndSectorCount, RndWipeSectors, WipeRnd, I: Integer;}
begin
//   RndSectorCount:= RndBetween(MinSectors, MaxSectors);
//   for I:= 1 to RndSectorCount do begin
//      RndWipeSectors:= RndBetween(0, SizeOf(AUnusedSectors)-1);
//      WipeRnd:= Unused[RndWipeSectors];
//      RndSector(Drive, WipeRnd);
//   end;
end;

function LizenzRec(Data: TSectorArray; Passwort: String): TLizenzRec;
var
   Ciph: TCipherManager;
   TempStr: String;
   TempList: TStringList;
begin
  try
   TempStr:= PChar(@Data[0]);
   Ciph:= TCipherManager.Create(Application);
//   Ciph.Algorithm:= 'BlowFish';
//   Ciph.Mode:= cmCTS;
   Ciph.InitKey(Passwort, NIL);
   TempStr:= Ciph.DecodeString(TempStr);
   Ciph.Free;
   TempList:= StrToList(TempStr, '|');

   if TempList.Count <> 15 then begin
      TempList.Free;
      result.Check:= -1;
      Exit;
   end;

   with Result do begin
      Check:=           StrToIntDef(TempList[0],  -3);
      Firma:=                       TempList[1];
      Vorname:=                     TempList[2];
      Name:=                        TempList[3];
      Zusatz:=                      TempList[4];
      Strasse:=                     TempList[5];
      PLZ:=                         TempList[6];
      Ort:=                         TempList[7];
      Land:=            StrToIntDef(TempList[8],  0);
      ProduktIndex:=    StrToIntDef(TempList[9],  0);
      LizenzIndex:=     StrToIntDef(TempList[10], 0);
      LizenzAnzahl:=    StrToIntDef(TempList[11], 0);
      FunktionsIndex:=  StrToIntDef(TempList[12], 0);
      Key1:=            StrToIntDef(TempList[13], 0);
      Key2:=                        TempList[14];
   end;
   TempList.Free;
  except
   result.Check:= -2;
  end;
end;

function ReadLizenz(var LizenzRec: TLizenzRec; Drive: Char; Sector: Integer; Passwort: String): Integer;
var
   Ciph: TCipherManager;
   DirDisk: TDirDisk;
   TempStr: String;
   TempList: TStringList;
   SData: TSectorArray;
begin
  try
   result:= 0;
   DirDisk:= TDirDisk.Create(Application);
   DirDisk.NeedWriteWarn:= FALSE;
   DirDisk.DriveName:= Drive;
   DirDisk.FirstSector:= Sector;
   DirDisk.Sectors:= 1;
   DirDisk.ReadSectors(SData);
   DirDisk.Free;

   TempStr:= PChar(@SData[0]);
   Ciph:= TCipherManager.Create(Application);
//   Ciph.Algorithm:= 'BlowFish';
//   Ciph.Mode:= cmCTS;
   Ciph.InitKey(Passwort, NIL);
   TempStr:= Ciph.DecodeString(TempStr);
   Ciph.Free;

   TempList:= StrToList(TempStr, '|');
   if TempList.Count <> 15 then begin
      result:= -1;
      TempList.Free;
      exit;
   end;
   with LizenzRec do begin
      Check:=           StrToIntDef(TempList[0],  -3);
      Firma:=                       TempList[1];
      Vorname:=                     TempList[2];
      Name:=                        TempList[3];
      Zusatz:=                      TempList[4];
      Strasse:=                     TempList[5];
      PLZ:=                         TempList[6];
      Ort:=                         TempList[7];
      Land:=            StrToIntDef(TempList[8],  0);
      ProduktIndex:=    StrToIntDef(TempList[9],  0);
      LizenzIndex:=     StrToIntDef(TempList[10], 0);
      LizenzAnzahl:=    StrToIntDef(TempList[11], 0);
      FunktionsIndex:=  StrToIntDef(TempList[12], 0);
      Key1:=            StrToIntDef(TempList[13], 0);
      Key2:=                        TempList[14];
   end;
   TempList.Free;
  except
   LizenzRec.Check:= 0;
   result:= -3;
  end;
end;

function WriteLizenz(LizenzRec: TLizenzRec; Drive: Char; Sector: Integer; Passwort: String): Integer;
var
   Ciph: TCipherManager;
   DirDisk: TDirDisk;
   TempStr: String;
   TempList: TStringList;
   SData: TSectorArray;
begin
  try
   result:= 0;
   TempList:= TStringList.Create;
   with TempList do begin
      with LizenzRec do begin
         Add(IntToStr(Check));
         Add(Firma);
         Add(Vorname);
         Add(Name);
         Add(Zusatz);
         Add(Strasse);
         Add(PLZ);
         Add(Ort);
         Add(IntToStr(Land));
         Add(IntToStr(Produktindex));
         Add(IntToStr(Lizenzindex));
         Add(IntToStr(Lizenzanzahl));
         Add(IntToStr(Funktionsindex));         
         Add(IntToStr(Key1));
         Add(Key2);
      end;
   end;
   TempStr:= ListToStr(TempList, '|');
   TempList.Free;
   Ciph:= TCipherManager.Create(Application);
//   Ciph.Algorithm:= 'BlowFish';
//   Ciph.Mode:= cmCTS;
   Ciph.InitKey(Passwort, NIL);
   TempStr:= Ciph.EncodeString(TempStr);
   Ciph.Free;
   DirDisk:= TDirDisk.Create(Application);
   DirDisk.NeedWriteWarn:= FALSE;   
   DirDisk.DriveName:= Drive;
   DirDisk.FirstSector:= Sector;
   DirDisk.Sectors:= 1;
   DirDisk.ReadSectors(SData);
   ChangeStrInArray(SData, TempStr, 0);
   DirDisk.WriteSectors(SData);
   DirDisk.Free;
  except
   LizenzRec.Check:= 0;
   result:= -1;
  end;
end;

function GeneratorRec(Data: TSectorArray; Passwort: String): TGeneratorRec;
var
   Ciph: TCipherManager;
   TempStr: String;
   TempList: TStringList;
begin
  try
   TempStr:= PChar(@Data[0]);
   Ciph:= TCipherManager.Create(Application);
//   Ciph.Algorithm:= 'BlowFish';
//   Ciph.Mode:= cmCTS;
   Ciph.InitKey(Passwort, NIL);
   TempStr:= Ciph.DecodeString(TempStr);
   Ciph.Free;
   TempList:= StrToList(TempStr, '|');

   if TempList.Count <> 10 then begin
      TempList.Free;
      result.Check:= -1;
      Exit;
   end;

   with Result do begin
      Check:=     StrToIntDef(TempList[0], -3);
      Firma:=     TempList[1];
      Zusatz:=    TempList[2];
      Strasse:=   TempList[3];
      PLZ:=       StrToIntDef(TempList[4], 0);
      Ort:=       TempList[5];
      Telefon:=   TempList[6];
      Fax:=       TempList[7];
      Email:=     TempList[8];
      Kontakt:=   TempList[9];
   end;
   TempList.Free;
  except
   result.Check:= -2;
  end;
end;

function ReadGenerator(var GeneratorRec: TGeneratorRec; Drive: Char;
                       Sector: Integer; Passwort: String): Integer;
var
   DirDisk: TDirDisk;
   SData: TSectorArray;
   Ciph: TCipherManager;
   TempList: TStringList;
   TempStr: String;
begin
  try
   result:= 0;
   DirDisk:= TDirDisk.Create(Application);
   DirDisk.NeedWriteWarn:= FALSE;
   DirDisk.DriveName:= Drive;
   DirDisk.FirstSector:= Sector;
   DirDisk.Sectors:= 1;
   DirDisk.ReadSectors(SData);
   DirDisk.Free;

   TempStr:= PChar(@SData[0]);

   Ciph:= TCipherManager.Create(Application);
   Ciph.InitKey(Passwort, NIL);
   TempStr:= Ciph.DecodeString(TempStr);
   Ciph.Free;

   TempList:= StrToList(TempStr, '|');
   if TempList.Count <> 10 then begin
      result:= -1;
      GeneratorRec.Check:= 0;
      exit;
   end;

   with GeneratorRec do begin
      Check:=     StrToIntDef(TempList[0], -3);
      Firma:=     TempList[1];
      Zusatz:=    TempList[2];
      Strasse:=   TempList[3];
      PLZ:=       StrToIntDef(TempList[4], 0);
      Ort:=       TempList[5];
      Telefon:=   TempList[6];
      Fax:=       TempList[7];
      Email:=     TempList[8];
      Kontakt:=   TempList[9];
   end;
   TempList.Free;

  except
   result:= -2;
  end;
end;

function WriteGenerator(GeneratorRec: TGeneratorRec; Drive: Char;
                        Sector: Integer; Passwort: String): Integer;
var
   DirDisk: TDirDisk;
   SData: TSectorArray;
   Ciph: TCipherManager;
   TempStr: String;
   TempList: TStringList;
begin
  try
   result:= 0;

   TempList:= TStringList.Create;
   with GeneratorRec do begin
      with TempList do begin
         Add(IntToStr(Check));
         Add(Firma);
         Add(Zusatz);
         Add(Strasse);
         Add(IntToStr(PLZ));
         Add(Ort);
         Add(Telefon);
         Add(Fax);
         Add(Email);
         Add(Kontakt);
      end;
   end;

   TempStr:= ListToStr(TempList, '|');
   TempList.Free;

   Ciph:= TCipherManager.Create(Application);
   Ciph.InitKey(Passwort, NIL);
   TempStr:= Ciph.EncodeString(TempStr);
   Ciph.Free;

   DirDisk:= TDirDisk.Create(Application);
   DirDisk.NeedWriteWarn:= FALSE;
   DirDisk.DriveName:= Drive;
   DirDisk.FirstSector:= Sector;
   DirDisk.Sectors:= 1;
   DirDisk.ReadSectors(SData);
   ChangeStrInArray(SData, TempStr, 0);
   DirDisk.WriteSectors(SData);
   DirDisk.Free;
  except
   result:= -2;
   GeneratorRec.Check:= 0;
  end;
end;

function SystemRec: TSystemRec;
var
   CPU: TCPU;
   Machine: TMachine;
begin
   with result do begin
      Machine:= TMachine.Create;
      Machine.GetInfo;
      BIOSDate:= Machine.BIOSDate;
      BIOSExtended:= Machine.BIOSExtendedInfo;
      BIOSName:= Machine.BIOSName;
      Machine.Free;
      CPU:= TCPU.Create;
      CPU.GetInfo;
      CPUSerialNumber:= CPU.SerialNumber;
      CPUVendorEx:= CPU.VendorEx;
      CPUVendorID:= CPU.VendorID;
      CPUCount:= CPU.Count;
      CPUFamily:= CPU.Family;
      CPUFrequency:= CPU.Frequency;
      CPUModel:= CPU.Model;
      CPUStepping:= CPU.Stepping;
      CPU.Free;
   end;
end;

function DefaultSystemRec: TSystemRec;
begin
   with result do begin
      BIOSDate:= NilString;
      BIOSExtended:= NilString;
      BIOSName:= NilString;
      CPUSerialNumber:= NilString;
      CPUVendorEx:= NilString;
      CPUVendorID:= NilString;
      CPUCount:= NilInteger;
      CPUFamily:= NilInteger;
      CPUFrequency:= NilInteger;
      CPUModel:= NilInteger;
      CPUStepping:= NilInteger;
   end;
end;

function ReadSystem(var SystemRec: TSystemRec; Drive: Char; Sector: Integer; Passwort: String): Integer;
var
   DirDisk: TDirDisk;
   SData: TSectorArray;
   Ciph: TCipherManager;
   TempStr: String;
   TempList: TStringList;
begin
  try
   result:= 0;

   DirDisk:= TDirDisk.Create(Application);
   with DirDisk do begin
      NeedWriteWarn:= FALSE;
      DriveName:= Drive;
      FirstSector:= Sector;
      Sectors:= 1;
      ReadSectors(SData);
      Free;
   end;

   TempStr:= PChar(@SData[0]);

   Ciph:= TCipherManager.Create(Application);
   with Ciph do begin
      InitKey(Passwort, NIL);
      TempStr:= DecodeString(TempStr);
      Free;
   end;

   TempList:= StrToList(TempStr, '|');
   with SystemRec do begin
      BIOSDate:=                    TempList[0];
      BIOSExtended:=                TempList[1];
      BIOSName:=                    TempList[2];
      CPUSerialNumber:=             TempList[3];
      CPUVendorEx:=                 TempList[4];
      CPUVendorID:=                 TempList[5];
      CPUCount:=        StrToIntDef(TempList[6],  0);
      CPUFamily:=       StrToIntDef(TempList[7],  0);
      CPUFrequency:=    StrToIntDef(TempList[8],  0);
      CPUModel:=        StrToIntDef(TempList[9],  0);
      CPUStepping:=     StrToIntDef(TempList[10], 0);
   end;
   TempList.Free;
  except
   result:= -1;
  end;
end;

function ReadSystemFromFile(var SystemRec: TSystemRec; Filename, Passwort: String): String;
var
   Ciph: TCipherManager;
   TempStr: String;
   TempList: TStringList;
begin
  try
   result:= '';
   if not FileExists(Filename) then exit;
   TempStr:= ReadStringFromFile(Filename);

   Ciph:= TCipherManager.Create(Application);
   with Ciph do begin
      InitKey(Passwort, NIL);
      TempStr:= DecodeString(TempStr);
      Free;
   end;

   TempList:= StrToList(TempStr, '|');
   with SystemRec do begin
      BIOSDate:=                    TempList[0];
      BIOSExtended:=                TempList[1];
      BIOSName:=                    TempList[2];
      CPUSerialNumber:=             TempList[3];
      CPUVendorEx:=                 TempList[4];
      CPUVendorID:=                 TempList[5];
      CPUCount:=        StrToIntDef(TempList[6],  0);
      CPUFamily:=       StrToIntDef(TempList[7],  0);
      CPUFrequency:=    StrToIntDef(TempList[8],  0);
      CPUModel:=        StrToIntDef(TempList[9],  0);
      CPUStepping:=     StrToIntDef(TempList[10], 0);
   end;
   TempList.Free;
  except
   result:= '';
  end;
end;

function WriteSystem(SystemRec: TSystemRec; Drive: Char; Sector: Integer;
            Passwort: String): Integer;
var
   DirDisk: TDirDisk;
   SData: TSectorArray;
   Ciph: TCipherManager;
   TempStr: String;
   TempList: TStringList;
begin
  try
   result:= 0;

   TempList:= TStringList.Create;
   with SystemRec do begin
      with TempList do begin
         Add(BIOSDate);
         Add(BIOSExtended);
         Add(BIOSName);
         Add(CPUSerialNumber);
         Add(CPUVendorEx);
         Add(CPUVendorID);
         Add(IntToStr(CPUCount));
         Add(IntToStr(CPUFamily));
         Add(IntToStr(CPUFrequency));
         Add(IntToStr(CPUModel));
         Add(IntToStr(CPUStepping));
      end;
   end;

   TempStr:= ListToStr(TempList, '|');
   TempList.Free;

   Ciph:= TCipherManager.Create(Application);
   with Ciph do begin
      InitKey(Passwort, NIL);
      TempStr:= EncodeString(TempStr);
      Free;
   end;

   DirDisk:= TDirDisk.Create(Application);
   with DirDisk do begin
      NeedWriteWarn:= FALSE;
      DriveName:= Drive;
      FirstSector:= Sector;
      Sectors:= 1;
      ReadSectors(SData);
      ChangeStrInArray(SData, TempStr, 0);
      WriteSectors(SData);
      Free;
   end;
  except
   result:= -2;
  end;
end;

function WriteSystemToFile(SystemRec: TSystemRec; Filename, Passwort: String): Integer;
var
   Ciph: TCipherManager;
   TempStr: String;
   TempList: TStringList;
begin
  try
   result:= 0;

   TempList:= TStringList.Create;
   with SystemRec do begin
      with TempList do begin
         Add(BIOSDate);
         Add(BIOSExtended);
         Add(BIOSName);
         Add(CPUSerialNumber);
         Add(CPUVendorEx);
         Add(CPUVendorID);
         Add(IntToStr(CPUCount));
         Add(IntToStr(CPUFamily));
         Add(IntToStr(CPUFrequency));
         Add(IntToStr(CPUModel));
         Add(IntToStr(CPUStepping));
      end;
   end;

   TempStr:= ListToStr(TempList, '|');
   TempList.Free;

   Ciph:= TCipherManager.Create(Application);
   with Ciph do begin
      InitKey(Passwort, NIL);
      TempStr:= EncodeString(TempStr);
      Free;
   end;

   SaveStringToFile(TempStr, Filename);

  except
   result:= -2;
  end;
end;

function CompareGeneratorRec(var GeneratorRec1, GeneratorRec2: TGeneratorRec): TGeneratorRecChecks;
begin
   ZeroMemory(@result, SizeOf(TGeneratorRecChecks));
   with GeneratorRec1 do begin
      if Firma <> GeneratorRec2.Firma then
         Include(result, failGRFirma);

      if Zusatz <> GeneratorRec2.Zusatz then
         Include(result, failGRZusatz);

      if Strasse <> GeneratorRec2.Strasse then
         Include(result, failGRStrasse);

      if PLZ <> GeneratorRec2.PLZ then
         Include(result, failGRPLZ);

      if Ort <> GeneratorRec2.Ort then
         Include(result, failGROrt);

      if Telefon <> GeneratorRec2.Telefon then
         Include(result, failGRTelefon);

      if Fax <> GeneratorRec2.Fax then
         Include(result, failGRFax);

      if Email <> GeneratorRec2.Email then
         Include(result, failGREmail);

      if Kontakt <> GeneratorRec2.Kontakt then
         Include(result, failGRKontakt);
   end;
end;

function CompareSystemRec(var SystemRec1, SystemRec2: TSystemRec): TSysRecChecks;
var
   I, resuSize: Integer;
begin
   ZeroMemory(@result, SizeOf(TSysRecChecks));
   with SystemRec1 do begin
      if (BIOSDate <> SystemRec2.BIOSDate) then
         Include(result, failBIOSDate);

      if (BIOSExtended <> SystemRec2.BIOSExtended) then
         Include(result, failBIOSExtended);

      if (BIOSName <> SystemRec2.BIOSName) then
         Include(result, failBIOSName);

      if (CPUSerialNumber <> SystemRec2.CPUSerialNumber) then
         Include(result, failCPUSerialNumber);

      if (CPUVendorEx <> SystemRec2.CPUVendorEx) then
         Include(result, failCPUVendorEx);

      if (CPUVendorID <> SystemRec2.CPUVendorID) then
         Include(result, failCPUVendorID);

      if (CPUCount <> SystemRec2.CPUCount) then
         Include(result, failCPUCount);

      if (CPUFamily <> SystemRec2.CPUFamily) then
         Include(result, failCPUFamily);

      if (CPUFrequency <> SystemRec2.CPUFrequency) then
         Include(result, failCPUFrequency);

      if (CPUModel <> SystemRec2.CPUModel) then
         Include(result, failCPUModel);

      if (CPUStepping <> SystemRec2.CPUStepping) then
         Include(result, failCPUStepping);
   end;
end;

function CompareLizenzRec(var LizenzRec1, LizenzRec2: TLizenzRec): TLizenzRecChecks;
begin
   ZeroMemory(@result, SizeOf(TLizenzRecChecks));
   with LizenzRec1 do begin
      if Firma <> LizenzRec2.Firma then
         Include(result, failLRFirma);

      if Vorname <> LizenzRec2.Vorname then
         Include(result, failLRVorname);

      if Name <> LizenzRec2.Name then
         Include(result, failLRName);

      if Zusatz <> LizenzRec2.Zusatz then
         Include(result, failLRZusatz);

      if Strasse <> LizenzRec2.Strasse then
         Include(result, failLRStrasse);

      if PLZ <> LizenzRec2.PLZ then
         Include(result, failLRPLZ);

      if Ort <> LizenzRec2.Ort then
         Include(result, failLROrt);

      if Land <> LizenzRec2.Land then
         Include(result, failLRLand);

      if Produktindex <> LizenzRec2.Produktindex then
         Include(result, failLRProduktindex);

      if Lizenzindex <> LizenzRec2.Lizenzindex then
         Include(result, failLRLizenzindex);

      if Lizenzanzahl <> LizenzRec2.Lizenzanzahl then
         Include(result, failLRLizenzanzahl);

      if Funktionsindex <> LizenzRec2.Funktionsindex then
         Include(result, failLRProduktindex);

      if Key1 <> LizenzRec2.Key1 then
         Include(result, failLRKey1);

      if Key2 <> LizenzRec2.Key2 then
         Include(result, failLRKey2);
   end;
end;

end.
