unit RegWorks;

interface

uses Windows, Registry, SysUtils, Classes, StringWorks;

type
   THostInfo = packed record
     Host:           String;
     Port:           Integer;
   end;

   TSysProxy = packed record
     FTP:            THostInfo;
     GOPHER:         THostInfo;
     HTTP:           THostInfo;
     HTTPS:          THostInfo;
     SOCKS:          THostInfo;
   end;

   REGW_INTERNETPROXY = class(TObject)
   private
      { Private-Deklarationen }
      function pIsProxyUse: Boolean;
      function pProxyString: String;
      function pProcessProxyString(ProxyString: String): TSysProxy;
   public
      { Public-Deklarationen }
      function Used:  Boolean;
      function Data: TSysProxy;
   end;

implementation

function REGW_INTERNETPROXY.pIsProxyUse: Boolean;
var
   Reg: TRegistry;
begin
   result:= FALSE;
   Reg:= TRegistry.Create(KEY_READ);
   with Reg do begin
      if OpenKey('Software\Microsoft\Windows\CurrentVersion\Internet settings',
         False) then result:= ReadBool('ProxyEnable');
      Free;
   end;
end;

function REGW_INTERNETPROXY.pProxyString: String;
var
   Reg: TRegistry;
begin
   result:= '';
   Reg:= TRegistry.Create(KEY_READ);
   with Reg do begin
      if OpenKey('Software\Microsoft\Windows\CurrentVersion\Internet settings',
         False) then result:= ReadString('ProxyServer');
      Free;
   end;
end;

function REGW_INTERNETPROXY.pProcessProxyString(ProxyString: String): TSysProxy;
var
   HostsPortList, HostPort: TStringList;
begin
   HostsPortList:= StrToList(ProxyString, ';');
   HostPort:= StrToList(HostsPortList[0], ':');
   result.FTP.Host:= StrToList(HostPort[0], '=')[1];
   result.FTP.Port:= StrToIntDef(HostPort[1], 0);

   HostPort:= StrToList(HostsPortList[1], ':');
   result.GOPHER.Host:= StrToList(HostPort[0], '=')[1];
   result.GOPHER.Port:= StrToIntDef(HostPort[1], 0);

   HostPort:= StrToList(HostsPortList[2], ':');
   result.HTTP.Host:= StrToList(HostPort[0], '=')[1];
   result.HTTP.Port:= StrToIntDef(HostPort[1], 0);

   HostPort:= StrToList(HostsPortList[3], ':');
   result.HTTPS.Host:= StrToList(HostPort[0], '=')[1];
   result.HTTPS.Port:= StrToIntDef(HostPort[1], 0);

   HostPort:= StrToList(HostsPortList[4], ':');
   result.SOCKS.Host:= StrToList(HostPort[0], '=')[1];
   result.SOCKS.Port:= StrToIntDef(HostPort[1], 0);
end;

function REGW_INTERNETPROXY.Used:  Boolean;
begin
   result:= pIsProxyUse;
end;

function REGW_INTERNETPROXY.Data: TSysProxy;
begin
   result:= pProcessProxyString(pProxyString);
end;

end.

