unit SysFolders;

interface

uses Registry, Windows, SysUtils;

function SYSF_APPDATA: String;
function SYSF_APPSDIR: String;
function SYSF_AUTOSTART: String;
function SYSF_BACKUPDIR: String;
function SYSF_BOOTDIR: String;
function SYSF_BOOTHOST: String;
function SYSF_CACHE: String;
function SYSF_COLORSHEMEDIR: String;
function SYSF_COMMONFILESDIR: String;
function SYSF_CONFIG: String;
function SYSF_COOKIES: String;
function SYSF_DESKTOP: String;
function SYSF_DEVICES: String;
function SYSF_FAVORITES: String;
function SYSF_FONTS: String;
function SYSF_HISTORY: String;
function SYSF_LOCALAPPDATA: String;
function SYSF_MACHINEDIR: String;
function SYSF_MEDIA: String;
function SYSF_MYPICTURES: String;
function SYSF_NETHOOD: String;
function SYSF_OLSPATH: String;
function SYSF_OTHERDEVICES: String;
function SYSF_PERSONAL: String;
function SYSF_PRINTHOOD: String;
function SYSF_PROGRAMFILESDIR: String;
function SYSF_PROGRAMFILESPATH: String;
function SYSF_PROGRAMSMENU: String;
function SYSF_RECENT: String;
function SYSF_SENDTO: String;
function SYSF_SHELLNEW: String;
function SYSF_STARTMENU: String;
function SYSF_UNINSTALLDIR: String;
function SYSF_WALLPAPER: String;
function SYSF_WINADMINDIR: String;
function SYSF_WINBOOTDIR: String;
function SYSF_WINCABS: String;
function SYSF_WINDIR: String;
function SYSF_WINSYSDIR: String;

function SYSF_MASTERPATH: String; (* Verzeichnis der Anwendung *)

function RegReadHKCU(Value: String): String;
function RegReadHKLM(Value: String): String;
function RegReadHKLMSetup(Value: String): String;

const FolderPathHKCU: String = '\Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders\';
const FolderPathHKLM: String = '\Software\Microsoft\Windows\CurrentVersion\';
const FolderPathHKLMSetup: String = '\Software\Microsoft\Windows\CurrentVersion\Setup\';
implementation

function SYSF_APPDATA: String;
begin
   result:= RegReadHKCU('AppData');
end;

function SYSF_CACHE: String;
begin
   result:= RegReadHKCU('Cache');
end;

function SYSF_COOKIES: String;
begin
   result:= RegReadHKCU('Cookies');
end;

function SYSF_DESKTOP: String;
begin
   result:= RegReadHKCU('Desktop');
end;

function SYSF_FAVORITES: String;
begin
   result:= RegReadHKCU('Favorites');
end;

function SYSF_FONTS: String;
begin
   result:= RegReadHKCU('Fonts');
end;

function SYSF_HISTORY: String;
begin
   result:= RegReadHKCU('History');
end;

function SYSF_LOCALAPPDATA: String;
begin
   result:= RegReadHKCU('Local AppData');
end;

function SYSF_MYPICTURES: String;
begin
   result:= RegReadHKCU('My Pictures');
end;

function SYSF_NETHOOD: String;
begin
   result:= RegReadHKCU('NetHood');
end;

function SYSF_PERSONAL: String;
begin
   result:= RegReadHKCU('Personal');
end;

function SYSF_PRINTHOOD: String;
begin
   result:= RegReadHKCU('PrintHood');
end;

function SYSF_PROGRAMSMENU: String;
begin
   result:= RegReadHKCU('Programs');
end;

function SYSF_RECENT: String;
begin
   result:= RegReadHKCU('Recent');
end;

function SYSF_SENDTO: String;
begin
   result:= RegReadHKCU('SendTo');
end;

function SYSF_STARTMENU: String;
begin
   result:= RegReadHKCU('Start Menu');
end;

function SYSF_AUTOSTART: String;
begin
   result:= RegReadHKCU('Startup');
end;

function SYSF_SHELLNEW: String;
begin
   result:= RegReadHKCU('Templates');
end;

function SYSF_WINDIR: String;
begin
   result:= RegReadHKLM('SystemRoot');
end;

function SYSF_MEDIA: String;
begin
   result:= RegReadHKLM('MediaPath');
end;

function SYSF_CONFIG: String;
begin
   result:= RegReadHKLM('ConfigPath');
end;

function SYSF_COMMONFILESDIR: String;
begin
   result:= RegReadHKLM('CommonFilesDir');
end;

function SYSF_DEVICES: String;
begin
   result:= RegReadHKLM('DevicePath');
end;

function SYSF_OTHERDEVICES: String;
begin
   result:= RegReadHKLM('OtherDevicePath');
end;

function SYSF_PROGRAMFILESDIR: String;
begin
   result:= RegReadHKLM('ProgramFilesDir');
end;

function SYSF_PROGRAMFILESPATH: String;
begin
   result:= RegReadHKLM('ProgramFilesPath');
end;

function SYSF_WALLPAPER: String;
begin
   result:= RegReadHKLM('WallPaperDir');
end;

function SYSF_BOOTDIR: String;
begin
   result:= RegReadHKLMSetup('BootDir');
end;

function SYSF_BOOTHOST: String;
begin
   result:= RegReadHKLMSetup('BootHost');
end;

function SYSF_COLORSHEMEDIR: String;
begin
   result:= RegReadHKLMSetup('ICMPath');
end;

function SYSF_MACHINEDIR: String;
begin
   result:= RegReadHKLMSetup('MachineDir');
end;

function SYSF_WINCABS: String;
begin
   result:= RegReadHKLMSetup('SourcePath');
end;

function SYSF_OLSPATH: String;
begin
   result:= RegReadHKLMSetup('OLSSrcPath');
end;

function SYSF_WINSYSDIR: String;
begin
   result:= RegReadHKLMSetup('SysDir');
end;

function SYSF_BACKUPDIR: String;
begin
   result:= RegReadHKLMSetup('BackupDir');
end;

function SYSF_APPSDIR: String;
begin
   result:= RegReadHKLMSetup('AppsDir');
end;

function SYSF_UNINSTALLDIR: String;
begin
   result:= RegReadHKLMSetup('UninstallDir');
end;

function SYSF_WINADMINDIR: String;
begin
   result:= RegReadHKLMSetup('WinAdminDir');
end;

function SYSF_WINBOOTDIR: String;
begin
   result:= RegReadHKLMSetup('WinBootDir');
end;

function SYSF_MASTERPATH: String;
begin
   result:= IncludeTrailingBackslash(ExtractFilePath(ParamStr(0)));
end;

//----------------------------------------------

function RegReadHKCU(Value: String): String;
var
   Reg: TRegistry;
begin
   Reg:= TRegistry.Create(KEY_READ);
   with Reg do begin
      RootKey:= HKEY_CURRENT_USER;
      OpenKey(FolderPathHKCU, FALSE);
      result:= IncludeTrailingBackslash(UpperCase(ReadString(Value)));
   end;
end;

function RegReadHKLM(Value: String): String;
var
   Reg: TRegistry;
begin
   Reg:= TRegistry.Create(KEY_READ);
   with Reg do begin
      RootKey:= HKEY_LOCAL_MACHINE;
      OpenKey(FolderPathHKLM, FALSE);
      result:= IncludeTrailingBackslash(UpperCase(ReadString(Value)));
   end;
end;

function RegReadHKLMSetup(Value: String): String;
var
   Reg: TRegistry;
begin
   Reg:= TRegistry.Create(KEY_READ);
   with Reg do begin
      RootKey:= HKEY_LOCAL_MACHINE;
      OpenKey(FolderPathHKLMSetup, FALSE);
      result:= IncludeTrailingBackslash(UpperCase(ReadString(Value)));
   end;
end;

end.

