unit USchmidt;

interface
  uses SysUtils, Controls, ExtCtrls, DBTables, Dialogs, StdCtrls, Mask, Classes,
       DbiProcs, DbiTypes, DbiErrs, Db{, NumEdit};

type
  TSetChar = Set of Char;
  TAMD     = array[1..3] of word;

const
  SC_NUM: TSetChar = [#8,'0'..'9'];

  { Valor por extenso de moedas }
  function Extenso(Valor : Extended): String;

  { Valor por extenso de taxas }
  function TaxaExt(Valor : Extended): String;

  { Criptografia binria }
  function CriptBin(S: string): String;

  { Verifica se o Cgc informado  valido }
  function  VerifyCgc(cCgc:String):Boolean;

  { Verifica se o Cpf informado  valido }
  function  VerifyCPF(s: String):Boolean;

  { Verifica se a data informada  valida }
  function  VerifyDate(text : TCaption):Boolean;

  { Attribui o digito verificador ao codigo informado }
  function  SetDigito(cCodigo, cFilial: integer): integer;

  { Verifica digito verificador do cdigo informado }
  function  VerifyDigito(cCodigo: String):Boolean;

  function  TodosIguais(s: String):Boolean;

  { Extrai somente os numeros de uma string }
  function  FilterNumber(cValor:String):String;

  { Tira espaos em brancas em uma string }
  function  Alltrim(cStr:String):String;

  function  LTrim(s: String):String;

  function  RTrim(s: String):String;

  { Adiciona zeros a esquerda }
  function  StrZero(n: Real;t:Byte):String;

  { Limpa (TEdit, TMaskEdit, TComboBox, TNumEdit) de um painel }
  procedure LimpaPanelEdt(p : TGroupBox);

  { Carrega os dados de uma tabela para a visualizao }
  procedure LoadFields(p: TGroupBox; t: TQuery);

  { Salva os dados contidos em um painel para uma tabela }
  procedure SaveFields(p: TGroupBox; t: TQuery);

  { Repete tantas quantas vezes o caracter informado }
  function  Repl(c: Char; tam: byte):String;

  function  PadR(st:String; t:byte; c: char):String;

  function  PadL(st:String; t:byte; c: char):String;

  function  FileNameToExt(Arq,Ext: String):String;

  function  pTransf(va: Longint; pos: Integer; variav: String): String;

  function  LastPos(S: Char; St: String):Byte;

  function  pSoNomeArq(cPath: String): String;

  function  pSoExtArq(cPath: String): String;

  function  AddMes(T:TDateTime;nMeses:Integer):TDateTime;

  function  AnoDe(d:TdateTime):Word;

  procedure pAMD(d: String; var amd: TAMD);

  function  UltDiaMes(Data: TDateTime):Word;

  function  pSubstituiVar(var1: String; var2, var3: Char): String;

  function  AnoBis(Ano: Integer): Boolean;

  function  Min(a,b : LongInt):LongInt;

  function  pFillChar(s, a: string; i, p: integer ): string;

  function  CalcTempoUtil( D1, D2: TDateTime): TDateTime;

  function  GetHours(T: TDateTime): string;

  function  CalcWeekEnds( Dt1, Dt2: TDateTime): integer;

  function  StrTimeToReal(s: String): Real;

  function  CalculaIdade(Data: TDateTime): Integer;

implementation

{ Valor por extenso de moedas
************************************************************************}
function Extenso(Valor : Extended): String;
var
  Centavos, Centena, Milhar, Milhao, Bilhao, Texto : string;
const
  Unidades: array [1..9] of string = ('um', 'dois', 'trs', 'quatro',
    'cinco',
    'seis', 'sete', 'oito',
    'nove');
  Dez : array [1..9] of string = ('onze', 'doze', 'treze',
    'quatorze', 'quinze',
    'dezesseis', 'dezessete',
    'dezoito', 'dezenove');
  Dezenas: array [1..9] of string = ('dez', 'vinte', 'trinta',
    'quarenta', 'cinquenta',
    'sessenta', 'setenta',
    'oitenta', 'noventa');
  Centenas: array [1..9] of string = ('cento', 'duzentos',
    'trezentos', 'quatrocentos',
    'quinhentos', 'seiscentos',
    'setecentos',
    'oitocentos', 'novecentos');

function ifs( Expressao: Boolean; CasoVerdadeiro, CasoFalso: String): String;
begin
  if Expressao then Result := CasoVerdadeiro else Result := CasoFalso;
end;

function MiniExtenso( Valor: String ): string;
var
  Unidade, Dezena, Centena: String;
begin

  Unidade := '';
  Dezena := '';
  Centena := '';

  if (Valor[2] = '1') and (Valor[3] <> '0') then begin
    Unidade := Dez[StrToInt(Valor[3])];
    Dezena := '';
  end else begin
    if Valor[2] <> '0' then
      Dezena := Dezenas[StrToInt(Valor[2])];
    if Valor[3] <> '0' then
      Unidade := Unidades[StrToInt(Valor[3])];
  end;

  if (Valor[1] = '1') and (Unidade = '') and (Dezena = '') then
    Centena := 'cem'
  else
    if Valor[1] <> '0' then Centena :=
      Centenas[StrToInt(Valor[1])]
    else Centena := '';

  Result := Centena +
  ifs( (Centena <> '') and ((Dezena <> '') or (Unidade <> '')), ' e ', '') + Dezena +
  ifs( (Dezena <> '') and (Unidade <> ''), ' e ', '') + Unidade;
end;

begin

  if Valor = 0 then begin
    Result := '';
    Exit;
  end;

  Texto := FormatFloat( '000000000000.00', Valor );
  Centavos := MiniExtenso( '0' + Copy( Texto, 14, 2 ) );
  Centena := MiniExtenso( Copy( Texto, 10, 3 ) );
  Milhar := MiniExtenso( Copy( Texto, 7, 3 ) );

  if Milhar <> '' then
    Milhar := Milhar + ' mil';
    Milhao := MiniExtenso( Copy( Texto, 4, 3 ) );

  if Milhao <> '' then
    Milhao := Milhao + ifs( Copy( Texto, 4, 3 ) = '001', ' milho',
      ' milhes');
    Bilhao := MiniExtenso( Copy( Texto, 1, 3 ) );

  if Bilhao <> '' then
    Bilhao := Bilhao + ifs( Copy( Texto, 1, 3 ) = '001', ' bilho',
      ' bilhes');

  if (Bilhao <> '') and (Milhao + Milhar + Centena = '') then
    Result := Bilhao + ' de reais'
  else if (Milhao <> '') and (Milhar + Centena = '') then
    Result := Milhao + ' de reais'
  else
    Result := Bilhao +
  ifs( (Bilhao <> '') and (Milhao + Milhar + Centena <> ''),
  ifs((Pos(' e ', Bilhao) > 0) or (Pos( ' e ', Milhao + Milhar + Centena ) > 0 ), ', ', ' e '), '') + Milhao +
  ifs( (Milhao <> '') and (Milhar + Centena <> ''),
  ifs((Pos(' e ', Milhao) > 0) or (Pos( ' e ', Milhar + Centena ) > 0 ), ', ', ' e '), '') + Milhar +
  ifs( (Milhar <> '') and (Centena <> ''),
  ifs(Pos( ' e ', Centena ) > 0, ', ', ' e '), '') + Centena + ifs( Int(Valor) = 1, ' real',
  ifs( Int(Valor)>1,' reais', '') );
  if Centavos <> '' then
    Result := Result + ifs(Centena <> '', ' e ', '') + Centavos +
    ifs( Copy( Texto, 14, 2 )= '01', ' centavo', ' centavos' );
end;



{ Valor por extenso de taxas
************************************************************************}
function TaxaExt(Valor : Extended): String;
var
  Centavos, Centena, Milhar, Milhao, Bilhao, Texto : string;
const
  Unidades: array [1..9] of string = ('um', 'dois', 'trs', 'quatro',
    'cinco',
    'seis', 'sete', 'oito',
    'nove');
  Dez : array [1..9] of string = ('onze', 'doze', 'treze',
    'quatorze', 'quinze',
    'dezesseis', 'dezessete',
    'dezoito', 'dezenove');
  Dezenas: array [1..9] of string = ('dez', 'vinte', 'trinta',
    'quarenta', 'cinquenta',
    'sessenta', 'setenta',
    'oitenta', 'noventa');
  Centenas: array [1..9] of string = ('cento', 'duzentos',
    'trezentos', 'quatrocentos',
    'quinhentos', 'seiscentos',
    'setecentos',
    'oitocentos', 'novecentos');

function ifs( Expressao: Boolean; CasoVerdadeiro, CasoFalso: String): String;
begin
  if Expressao then Result := CasoVerdadeiro else Result := CasoFalso;
end;

function MiniExtenso( Valor: String ): string;
var
  Unidade, Dezena, Centena: String;
begin

  Unidade := '';
  Dezena := '';
  Centena := '';

  if (Valor[2] = '1') and (Valor[3] <> '0') then begin
    Unidade := Dez[StrToInt(Valor[3])];
    Dezena := '';
  end else begin
    if Valor[2] <> '0' then
      Dezena := Dezenas[StrToInt(Valor[2])];
    if Valor[3] <> '0' then
      Unidade := Unidades[StrToInt(Valor[3])];
  end;

  if (Valor[1] = '1') and (Unidade = '') and (Dezena = '') then
    Centena := 'cem'
  else
    if Valor[1] <> '0' then Centena :=
      Centenas[StrToInt(Valor[1])]
    else
      Centena := '';
      Result := Centena +
      ifs( (Centena <> '') and ((Dezena <> '') or (Unidade <> '')), ' e ', '') + Dezena +
      ifs( (Dezena <> '') and (Unidade <> ''), ' e ', '') + Unidade;
end;

begin

  if Valor = 0 then begin
    Result := '';
    Exit;
  end;

  Texto := FormatFloat( '000000000000.00', Valor );
  Centavos := MiniExtenso( '0' + Copy( Texto, 14, 2 ) );
  Centena := MiniExtenso( Copy( Texto, 10, 3 ) );
  Milhar := MiniExtenso( Copy( Texto, 7, 3 ) );

  if Milhar <> '' then
    Milhar := Milhar + ' mil';
    Milhao := MiniExtenso( Copy( Texto, 4, 3 ) );

  if Milhao <> '' then
    Milhao := Milhao + ifs( Copy( Texto, 4, 3 ) = '001', ' milho',
      ' milhes');
    Bilhao := MiniExtenso( Copy( Texto, 1, 3 ) );

  if Bilhao <> '' then
    Bilhao := Bilhao + ifs( Copy( Texto, 1, 3 ) = '001', ' bilho',
      ' bilhes');

  if (Bilhao <> '') and (Milhao + Milhar + Centena = '') then
    Result := Bilhao + ' de inteiros'
  else if (Milhao <> '') and (Milhar + Centena = '') then
    Result := Milhao + ' de inteiros'
  else
    Result := Bilhao +
  ifs( (Bilhao <> '') and (Milhao + Milhar + Centena <> ''),
  ifs((Pos(' e ', Bilhao) > 0) or (Pos( ' e ', Milhao + Milhar + Centena ) > 0 ), ', ', ' e '), '') + Milhao +
  ifs( (Milhao <> '') and (Milhar + Centena <> ''),
  ifs((Pos(' e ', Milhao) > 0) or (Pos( ' e ', Milhar + Centena ) > 0 ), ', ', ' e '), '') + Milhar +
  ifs( (Milhar <> '') and (Centena <> ''),
  ifs(Pos( ' e ', Centena ) > 0, ', ', ' e '), '') + Centena + ifs( Int(Valor) = 1, ' inteiro',
  ifs( Int(Valor) > 1,' inteiros', '') );
  if Centavos <> '' then
    Result := Result + Ifs(Centena <> '', ' e ', '') + Centavos +
    ifs( Copy( Texto, 14, 2 )= '01', ' centesimo', ' centesimos' );
end;



{ Criptografia binria
************************************************************************}
function CriptBin( S: string ): string;
var
  X, K: integer;
  Q: PChar;
  R, cBin, cHex: string;

function StrToBin(a: String): PChar;
  var
    i, j: Integer;
    s: String;
    p, r: PChar;
  const
   HexString: array[0..15] of Char = ('0', '1', '2', '3',
                                       '4', '5', '6', '7',
                                       '8', '9', 'A', 'B',
                                       'C', 'D', 'E', 'F');
    BinString: array[0..15] of String = ('0000', '0001', '0010', '0011',
                                         '0100', '0101', '0110', '0111',
                                         '1000', '1001', '1010', '1011',
                                         '1100', '1101', '1110', '1111');
  begin
    s := '';
   r := StrAlloc(65000);
    p := StrAlloc(65000);
    StrPCopy(r, '');
    for i := 1 to Length(a) do
      s := s + IntToHex(Ord(a[i]), 2);
    for i := 1 to Length(s) do begin
     for j := 0 to 15 do begin
       if s[i] = HexString[j] then begin
     StrPCopy(p, '');
          StrPCopy(p, BinString[j]);
          StrCat(r, p);
        end;
      end;
    end;
    StrDispose(p);
    Result := StrAlloc(65000);
    StrCopy(Result, r);
  end;

  function BinToStr(a: PChar): String;
  var
   i, j: Integer;
    s: String;
    HexString: array[0..15] of Char;
    BinString: array[0..15] of String;
  begin
   HexString[0] := '0';
    HexString[1] := '1';
    HexString[2] := '2';
    HexString[3] := '3';
    HexString[4] := '4';
    HexString[5] := '5';
    HexString[6] := '6';
    HexString[7] := '7';
    HexString[8] := '8';
    HexString[9] := '9';
    HexString[10] := 'A';
    HexString[11] := 'B';
    HexString[12] := 'C';
    HexString[13] := 'D';
    HexString[14] := 'E';
    HexString[15] := 'F';
    BinString[0] := '0000';
    BinString[1] := '0001';
    BinString[2] := '0010';
    BinString[3] := '0011';
    BinString[4] := '0100';
    BinString[5] := '0101';
    BinString[6] := '0110';
    BinString[7] := '0111';
    BinString[8] := '1000';
    BinString[9] := '1001';
    BinString[10] := '1010';
    BinString[11] := '1011';
    BinString[12] := '1100';
    BinString[13] := '1101';
    BinString[14] := '1110';
    BinString[15] := '1111';
    s := '';
    Result := '';
    j := 0;
    while a[j] <> #0 do begin
     i := 0;
   s := '';
     while i < 4 do begin
       s := s + a[j];
        inc(i);
        inc(j);
      end;
      for i := 0 to 15 do begin
       if s = BinString[i] then begin
          Result := Result + HexString[i];
        end;
      end;
    end;
  end;

begin
  cBin := StrPas( StrToBin( S ) );

  for X := 1 to length( cBin ) do begin
    if (X mod 3) = 0 then begin
      if cBin[X] = '1' then
        cBin[X] := '0'
      else
        cBin[X] := '1';
    end;
  end;

  Q := StrAlloc( 65000 );
  StrPCopy( Q, cBin );
  cHex := BinToStr( Q );
  StrDispose( Q );
  X := 1;
  R := '';
  while X <= Length( cHex ) do begin
    K := StrToInt( '$' + Copy( cHex, X, 2 ) );
    R := R + Chr( K );
    Inc( X, 2 );
  end;

  Result := R;
end;



Function VerifyCgc(cCgc:String):Boolean;
var
  lCgc: Boolean;
  i, j, Soma, Digito: Integer;
  Cgc1, Cgc2, Mult, Controle: String;
begin
Digito := 0; 
cCgc := FilterNumber(cCgc);
 if length(cCgc) = 14 then begin
  Cgc1 := Copy(cCgc,1,12);
  Cgc2 := Copy(cCgc,13,2);
  Mult := '543298765432';
  Controle := '';
  for j := 1 to 2 do
    begin
      Soma := 0;
      for i := 1 to 12 do
        begin
          Soma := Soma + StrToInt(Cgc1[i]) * StrToInt(Mult[i]);
        end;
      if j = 2 then
        Soma := Soma + (2 * Digito);

      Digito := (Soma * 10) mod 11;

      if Digito = 10 then
        Digito := 0;

      Controle := Controle + IntToStr(Digito);

      Mult := '654329876543';
    end;

    if (cCgc <> '') and (Controle <> Cgc2) then
      lCgc := False
    else
      lCgc := True;
 end else
    lCGC := False;
    Result := lCgc and (Not TodosIguais(cCGC));
end;



Function VerifyCPF(s: String):Boolean;
var
    resto : byte;
    soma  : Integer;
    i, dv : Byte;
    f     : Byte;
    ok    : Boolean;
begin
    s  := FilterNumber(s);
    ok := (Length(s) = 11) and Not TodosIguais(s);
    if ok then begin
       f := 2;
       soma := 0;
       for i := 9 downto 1 do begin
          soma := soma + StrToInt(s[i]) * f;
          f := f + 1;
       end;
       resto := soma mod 11;
       if resto < 2 then
          dv := 0
       else
          dv := 11 - resto;
       ok := (dv = StrToInt(s[10]));
       if ok then begin
          f := 2;
          soma := 0;
          for i := 10 downto 1 do begin
             soma := soma + StrToInt(s[i]) * f;
             f    := f + 1;
          end;
          resto := soma mod 11;
          if resto < 2 then
              dv := 0
          else
              dv := 11 - resto;
          ok := (dv = StrToInt(s[11]));
       End;
    End;
    VerifyCPF := ok;
End;


Function VerifyDate(text : TCaption):Boolean;
const
    aMes : Array[1..12] of byte = (31,28,31,30,31,30,31,31,30,31,30,31);
var
    ok    : Boolean;
    d,m,a : Integer;
    cD,cM,cA : String[4];
    sDate : String;
begin
    sDate := text;
    ok := Length(sDate) = 10;
    if ok then begin
         ok := (sDate[3] = '/') and (sDate[6] = '/');
         cD := Alltrim(copy(sDate,1,2));
         cM := Alltrim(copy(sDate,4,2));
         cA := Alltrim(copy(sDate,7,4));
         if ok then
              ok := ( length(cd) = 2 ) and ( length(cm) = 2 ) and ( length(ca) = 4 );
         if ok then begin
              d := StrToIntDef(cd,-1);
              m := StrToIntDef(cm,-1);
              a := StrToIntDef(ca,-1);
              ok := (m > 0) and (m < 13) and (d > 0) and (d < aMes[m]+1);
              if ok and (a div 4 = 0) then begin
                   ok := d < 30;
              end;
         End;
    end;
    if Not Ok Then
         MessageDlg('Data Invlida!',mtWarning,[mbOk],0);
    VerifyDate := ok;
end;



function SetDigito( cCodigo, cFilial: integer ): integer;
var
  Soma, I: Integer;
  Peso, Dig: Byte;
  cCod: string;
begin
  cCod := IntToStr( cCodigo )+IntToStr( cFilial );
  Soma := 0;
  Peso := 2;
  for I := Length( cCod ) downto 1 do begin
    Soma := Soma + StrToInt( cCod[I] ) * Peso;
    if Peso < 9 then Peso := Peso + 1 else Peso := 2;
  end;
  Soma := Soma * 10;
  Dig := ( Soma mod 11 ) mod 10;
  SetDigito := StrToInt( cCod + IntToStr(Dig) );
end;



function VerifyDigito(cCodigo: string): Boolean;
var
  ok : Boolean;
  soma : LongInt;
  i    : Byte;
  dig  : Byte;
  peso : byte;
  digR : byte;
begin
  cCodigo := FilterNumber(cCodigo);
  cCodigo := PadL(cCodigo,9,'0');
  ok := (length(cCodigo) = 9);
  if ok then begin
    dig    := StrToInt(cCodigo[length(cCodigo)]);
    cCodigo := Copy(cCodigo,1,length(cCodigo)-1);
    soma := 0;
    peso := 2;
    for i := length(cCodigo) downto 1 do begin
         soma := soma + StrToInt(cCodigo[i]) * peso;
         if peso < 9 then peso := peso + 1
                     else peso := 2;
    end;
    soma := soma * 10;
    digR := (soma mod 11) mod 10;
    ok := (digR = dig);
  end;
  VerifyDigito := ok
end;



Function TodosIguais(s: String):Boolean;
var
   i     : Byte;
   igual : Boolean;
begin
   igual := True;
   i := 2;
   while igual and (i <= length(s)) do begin
       igual := (s[i] = s[1]);
       i := i + 1;
   end;
   TodosIguais := igual;
End;



Function Alltrim(cStr:String):String;
var
  cCopy: String;
  i: Integer;
begin
  cCopy := '';
  for i := 1 to Length(cStr) do
    begin
      if cStr[i] <> ' ' then
        cCopy := cCopy + cStr[i];
    end;
  Result := cCopy;
end;



Function  LTrim(s: String):String;
begin
    while (s <> '') and (s[1] = '') do
         s := copy(s,2,length(s)-1);
    LTrim := s;
End;


Function  RTrim(s: String):String;
begin
    while (s <> '') and (s[Length(s)] = ' ') do
         s := copy(s,1,length(s)-1);
    RTrim := s;
End;


Function FilterNumber(cValor:String):String;
const
    SC_NUM : TSetChar = ['0','1','2','3','4','5','6','7','8','9'];
var
    i : byte;
    cNew : String;
begin
    cNew := '';
    for i := 1 to length(cValor) do
         if cValor[i] in SC_NUM then
              cNew := cNew + cValor[i];
    FilterNumber := cNew;
End;



Function StrZero(n:Real;t:byte):String;
var
    s : String;
    i : Integer;
begin
    str(n:t:0,s);
    for i := 1 to length(s) do
         if s[i] = ' ' then
              s[i] := '0';
    StrZero := s;
End;

 

Procedure LimpaPanelEdt(p : TGroupBox);
var
  i : Integer;
begin
{  with p do
    for i := 0 to ControlCount - 1 do begin
      if (Controls[i] is TEdit) then
        TEdit(Controls[i]).Text := '';
      if (Controls[i] is TMaskEdit) then
        TMaskEdit(Controls[i]).Text := '';
      if (Controls[i] is TNumEdit) then
        TNumEdit(Controls[i]).Value := 0;
      if (Controls[i] is TComboBox) then
        if TComboBox(Controls[i]).Style = csDropDown then
          TComboBox(Controls[i]).Text := ''
        else if TComboBox(Controls[i]).Style = csDropDownList then
          TComboBox(Controls[i]).ItemIndex := -1;
      if (Controls[i] is TCheckBox) then
        TCheckBox(Controls[i]).Checked := False;
  end; }
end;

  

procedure LoadFields(p: TGroupBox; t: TQuery);
var
  i : Integer;
begin
(*  with p do begin
    for i := 0 to ControlCount - 1 do begin
      { Edit }
      if (Controls[i] is TEdit) then
        TEdit(Controls[i]).Text := t.FieldByName(TEdit(Controls[i]).Name).AsString;

      { MaskEdit }
      if (Controls[i] is TMaskEdit) then
        TMaskEdit(Controls[i]).Text := t.FieldByName(TMaskEdit(Controls[i]).Name).AsString;

      { NumEdit }
      if (Controls[i] is TNumEdit) then
          TNumEdit(Controls[i]).Value := t.FieldByName(TNumEdit(Controls[i]).Name).AsFloat;

      { ComboBox }
      if (Controls[i] is TComboBox) then
        if TComboBox(Controls[i]).Style = csDropDown then
          TComboBox(Controls[i]).Text := t.FieldByName(TComboBox(Controls[i]).Name).AsString
        else if TComboBox(Controls[i]).Style = csDropDownList then
          TComboBox(Controls[i]).ItemIndex := t.FieldByName(TComboBox(Controls[i]).Name).AsInteger;

    end;
  end; *)
end;

  

procedure SaveFields(p: TGroupBox; t: TQuery);
var
  i : Integer;
begin
(*  with p do begin
    for i := 0 to ControlCount - 1 do begin
      { Edit }
      if (Controls[i] is TEdit) then
        t.FieldByName(TEdit(Controls[i]).Name).AsString := TEdit(Controls[i]).Text;

      { MaskEdit }
      if (Controls[i] is TMaskEdit) then
        t.FieldByName(TMaskEdit(Controls[i]).Name).AsString := TMaskEdit(Controls[i]).Text;

      { NumEdit }
      if (Controls[i] is TNumEdit) then
        t.FieldByName(TNumEdit(Controls[i]).Name).AsFloat := TNumEdit(Controls[i]).Value;

      { ComboBox }
      if (Controls[i] is TComboBox) then
        if TComboBox(Controls[i]).Style = csDropDown then
          t.FieldByName(TComboBox(Controls[i]).Name).AsString := TComboBox(Controls[i]).Text
        else if TComboBox(Controls[i]).Style = csDropDownList then
          t.FieldByName(TComboBox(Controls[i]).Name).AsInteger := TComboBox(Controls[i]).ItemIndex;
    end;
  end; *)
end;

  

function Repl(c: Char; tam: byte):String;
var
    i : Byte;
    st : String;
begin
    st := '';
    for i := 1 to tam do
         st := st + c;
    repl := st;
End;

  

Function PadR(st:String; t:byte; c: char):String;
begin
    while length(st) < t do
         st := st + c;
    st := copy(st,1,t);
    PadR := st;
End;

  

Function PadL(st:String; t:byte; c: char):String;
begin
    while length(st) < t do
         st := c + st;
    st := copy(st,1,t);
    PadL := st;
End;

  

Function FileNameToExt(Arq,Ext: String):String;
Var
    P : Byte;
begin
    P := LastPos('.',Arq);
    if p = 0 then
         Arq := Arq + '.' + Ext
    else
         Arq := Copy(Arq,1,p)+Ext;
    FileNameToExt := Arq;
End;

  

Function LastPos(S: Char; St: String):Byte;
Var
    p : Byte;
    achou : Boolean;
Begin
    p := Length(st);
    achou := False;
    while (P > 0) and (Not Achou) do
         if st[p] = s then
              achou := True
         else
              p := p - 1;
    LastPos := p;
End;

  

function pSoNomeArq(cPath: String): String;
var
  p1: Integer;
begin
  p1 := LastPos('\',cPath);
  if p1 = 0 then
    pSoNomeArq := ''
  else
    pSoNomeArq := Copy(cPath,p1+1,Length(cPath));
end;

  

function pSoExtArq(cPath: String): String;
var
  p1: Integer;
begin
  p1 := LastPos('\',cPath);
  if p1 = 0 then
    pSoExtArq := ''
  else
    pSoExtArq := Copy(cPath,1,p1-1);
end;

  

Function AddMes(T:TDateTime;nMeses:Integer):TDateTime;
Var
    d,m,a : Word;
    erro  : Boolean;
begin
    DecodeDate(t,a,m,d);
    m := m + nMeses;
    while m > 12 do begin
         m := m - 12;
         a := a + 1;
    End;
    while m < 1 do begin
         m := 12 + m;
         a := a - 1;
    end;
    {erro := False;}
    Repeat
       Try
         AddMes := EncodeDate(a,m,d);
         Erro := False;
       Except
         Erro := True;
         D := D - 1;
         AddMes := EncodeDate(a,m,d);
       end;
    Until Not Erro
End;

  

Function AnoDe(d:TdateTime):Word;
var
    a,m,dia : Word;
begin
    DecodeDate(d,a,m,dia);
    AnoDe := a;
End;

  

procedure pAMD(d: String; var amd:TAMD);
begin
  amd[1] := StrToInt(Copy(d, 7, 4));
  amd[2] := StrToInt(Copy(d, 4, 2));
  amd[3] := StrToInt(Copy(d, 1, 2));
end;

  

function UltDiaMes(Data: TDateTime):Word;
const
  aMes : Array[1..12] of byte = (31,28,31,30,31,30,31,31,30,31,30,31);
var
  pAno, pMes, pDia: Word;
begin
  DecodeDate(Data, pAno, pMes, pDia);
  if pMes = 2 then
    if AnoBis(pAno) then
      UltDiaMes := aMes[pMes] + 1
    else
      UltDiaMes := aMes[pMes]
  else
    UltDiaMes := aMes[pMes];
end;

  

function pTransf(va: Longint; pos: Integer; Variav: String): String;
var
  ini, x, y, i: Integer;
  v: String;
begin
  v := IntToStr(va);
  x := Length(v) div pos;
  y := Length(v) mod pos;

  if y <> 0 then begin
    Insert(Variav, v, y+1);
    Ini := y + 1;
  end else
    Ini := 1;

  for i := 1 to x - 1 do begin
    Insert(Variav, v, Ini + Pos + 1);
    Ini := Ini + Pos + 1;
  end;

  pTransf := v;

end;

  

function pSubstituiVar(var1: String; var2, var3: Char): String;
var
  i: Integer;
begin

  for i := 1 to Length(var1) do begin
    if var1[i] = var2 then
      var1[i] := var3;
  end;

  Result := var1;

end;

  

function AnoBis(Ano: Integer): Boolean;
begin
  if (Ano mod 4 <> 0) then
    AnoBis := False
  else
    if (Ano mod 100 <> 0) then
      AnoBis := True
    else
    if (Ano mod 400 <> 0) then
      AnoBis := False
    else
      AnoBis := True;
end;

  

Function Min(a,b : LongInt):LongInt;
begin
    if a < b then Min := a
             else Min := b;
end;

  

function pFillChar(s, a: string; i, p: integer ): string;
begin
  while Length(s) < i do begin
    if p = 0 then
      s := s + a
    else
      s := a + s;
  end;
  pFillChar := s;
end;

  

function GetHours(T: TDateTime): string;
{Transforma TDateTime em nmero de horas}
var
  i: real;
begin
  i := 24 * T; {Multiplica por 24 p/ obter nmero de horas}
  Result := IntToStr(Trunc(i)) + ':' + {Horas}
            FormatFloat('00', Trunc(Frac(i)*60+0.01)) + ':' + {Minutos}
            FormatFloat('00', Trunc(Frac(Frac(i)*60+0.01)*60)); {Segundos}
end;

  

function CalcWeekEnds( Dt1, Dt2: TDateTime): integer;
var
  I, Dias, n: integer;
  a, m, d: word;
  Data: TDateTime;
begin
  n := 0;
  Dias := Trunc(Dt2) - Trunc(Dt1);
  DecodeDate(Dt1, a, m, d);
  for I := 0 to Dias -1 do begin
    try
      Data := EncodeDate(a, m, d+I);
    except
      try
        Data := EncodeDate(a, m+1, 1);
      except
        Data := EncodeDate(a+1, 1, 1);
      end;
    end;
    if not (DayOfWeek(Data) in [2..6]) then
      n := n + 1;
  end;
  CalcWeekEnds := n;
end;

  

function CalcTempoUtil( D1, D2: TDateTime): TDateTime;
var
  Dif, WeekEnds, I: Integer;
  Diferenca: Real;
  H1, H2: TDateTime;
begin
  // Calcula a diferena em dias
  Dif := Trunc(D2) - Trunc(D1);
  if Dif = 0 then begin
    // Calcula somente a diferena em HORAS
    CalcTempoUtil := (D2-D1);
  end else begin
    // Calcula se a diferena  maior que 24 horas
    Diferenca := Trunc(D2 - D1);
    if Diferenca = 0 then begin
      // Calcula a diferena em dias menos total horas no teis sem o intervalo do meio dia
      CalcTempoUtil := Diferenca - StrToTime('14:00:00');
    end else begin
      H1 := Frac(D1);
      H2 := Frac(D2);
      WeekEnds := CalcWeekEnds(D1, D2);
      Diferenca := (D2 - D1) - WeekEnds;

      for I := 1 to Trunc(Diferenca) do 
        Diferenca := Diferenca - StrToTime('15:30:00');

      if H2 < H1 then
        Diferenca := Diferenca - StrToTime('14:00:00');

      CalcTempoUtil := Diferenca;
    end;
  end;
end;



function StrTimeToReal(s: String): Real;
var
  n: integer;
begin
  n := Pos(':',s);
  if n > 0 then begin
    s := Copy(s,1,n+2);
    s[n] := ',';
    StrTimeToReal := StrToFloat(s);
  end else
    StrTimeToReal := 0.0;
end;


function CalculaIdade(Data: TDateTime): Integer;
var
  a1,m1,d1, a2, m2, d2: word;
  Idade: Integer;
begin
  DecodeDate(Data, a1, m1, d1);
  DecodeDate(Date, a2, m2, d2);
  Idade := a2 - a1;
  if m2 < m1 then
  Idade := Idade - 1
  else if m2 = m1 then begin
   if d2 < d1 then
   Idade := Idade - 1;
 end;
 CalculaIdade := Idade;
end;

end.




